LOCAL_PATH:= $(call my-dir)

include $(CLEAR_VARS)

# Force permissive domains to be unconfined+enforcing?
#
# During development, this should be set to false.
# Permissive means permissive.
#
# When we're close to a release and SELinux new policy development
# is frozen, we should flip this to true. This forces any currently
# permissive domains into unconfined+enforcing.
#
FORCE_PERMISSIVE_TO_UNCONFINED:=false
USERDEBUG_OR_ENG:=true

ifeq ($(TARGET_BUILD_VARIANT),user)
  # User builds are always forced unconfined+enforcing
  FORCE_PERMISSIVE_TO_UNCONFINED:=true
  # For CIL this is a policy tunable
  USERDEBUG_OR_ENG:=false
endif


# SELinux policy version.
# Must be <= /selinux/policyvers reported by the Android kernel.
# Must be within the compatibility range reported by checkpolicy -V.
POLICYVERS ?= 26

MLS_SENS=1
MLS_CATS=1024

# Quick edge case error detection for BOARD_CIL_SEPOLICY_REPLACE.
# Builds the singular path for each replace file.
sepolicy_replace_paths :=
$(foreach pf, $(BOARD_CIL_SEPOLICY_REPLACE), \
  $(if $(filter $(pf), $(BOARD_CIL_SEPOLICY_UNION)), \
    $(error Ambiguous request for sepolicy $(pf). Appears in both \
      BOARD_CIL_SEPOLICY_REPLACE and BOARD_CIL_SEPOLICY_UNION), \
  ) \
  $(eval _paths := $(filter-out $(BOARD_CIL_SEPOLICY_IGNORE), \
  $(wildcard $(addsuffix /$(pf), $(BOARD_CIL_SEPOLICY_DIRS))))) \
  $(eval _occurrences := $(words $(_paths))) \
  $(if $(filter 0,$(_occurrences)), \
    $(error No sepolicy file found for $(pf) in $(BOARD_CIL_SEPOLICY_DIRS)), \
  ) \
  $(if $(filter 1, $(_occurrences)), \
    $(eval sepolicy_replace_paths += $(_paths)), \
    $(error Multiple occurrences of replace file $(pf) in $(_paths)) \
  ) \
  $(if $(filter 0, $(words $(wildcard $(addsuffix /$(pf), $(LOCAL_PATH))))), \
    $(error Specified the sepolicy file $(pf) in BOARD_CIL_SEPOLICY_REPLACE, \
      but none found in $(LOCAL_PATH)), \
  ) \
)

# Quick edge case error detection for BOARD_CIL_SEPOLICY_UNION.
# This ensures that a requested union file exists somewhere
# in one of the listed BOARD_CIL_SEPOLICY_DIRS.
$(foreach pf, $(BOARD_CIL_SEPOLICY_UNION), \
  $(if $(filter 0, $(words $(wildcard $(addsuffix /$(pf), $(BOARD_CIL_SEPOLICY_DIRS))))), \
    $(error No sepolicy file found for $(pf) in $(BOARD_CIL_SEPOLICY_DIRS)), \
  ) \
)

# Builds paths for all requested policy files w.r.t
# both BOARD_CIL_SEPOLICY_REPLACE and BOARD_CIL_SEPOLICY_UNION
# product variables.
# $(1): the set of policy name paths to build
build_policy = $(foreach type, $(1), \
  $(filter-out $(BOARD_CIL_SEPOLICY_IGNORE), \
    $(foreach expanded_type, $(notdir $(wildcard $(addsuffix /$(type), $(LOCAL_PATH)))), \
      $(if $(filter $(expanded_type), $(BOARD_CIL_SEPOLICY_REPLACE)), \
        $(wildcard $(addsuffix $(expanded_type), $(sort $(dir $(sepolicy_replace_paths))))), \
        $(LOCAL_PATH)/$(expanded_type) \
      ) \
    ) \
    $(foreach union_policy, $(wildcard $(addsuffix /$(type), $(BOARD_CIL_SEPOLICY_DIRS))), \
      $(if $(filter $(notdir $(union_policy)), $(BOARD_CIL_SEPOLICY_UNION)), \
        $(union_policy), \
      ) \
    ) \
  ) \
)


########## Build CIL Policy and install as sepolicy ############
include $(CLEAR_VARS)

LOCAL_MODULE := cil_sepolicy
LOCAL_MODULE_CLASS := ETC
LOCAL_MODULE_TAGS := optional

include $(BUILD_SYSTEM)/base_rules.mk

cil_file_contexts := $(intermediates)/file_contexts
cil_tunables := $(intermediates)/tunables.cil

cil_policy_modules := $(call build_policy, *.cil)

$(LOCAL_BUILT_MODULE) : $(cil_policy_modules) $(HOST_OUT_EXECUTABLES)/secilc
	@mkdir -p $(dir $@)
	echo "; Auto generated tunables" > $(cil_tunables)
	echo "(tunable recovery_only false)" >> $(cil_tunables)
	echo "(tunable force_permissive_to_unconfined $(FORCE_PERMISSIVE_TO_UNCONFINED))" >> $(cil_tunables)
	echo "(tunable userdebug_or_eng $(USERDEBUG_OR_ENG))" >> $(cil_tunables)
	$(hide) $(HOST_OUT_EXECUTABLES)/secilc -c $(POLICYVERS) -f $(cil_file_contexts) -o $@ $(cil_policy_modules) $(cil_tunables)
	$(hide) $(HOST_OUT_EXECUTABLES)/secilc --disable-dontaudit -c $(POLICYVERS) -o $@.disable-dontaudit  $(cil_policy_modules) $(cil_tunables)
	@mkdir -p $(TARGET_ROOT_OUT)
	@cp $@ $(TARGET_ROOT_OUT)/sepolicy
	@cp $(cil_file_contexts) $(TARGET_ROOT_OUT)/file_contexts

built_sepolicy := $(LOCAL_BUILT_MODULE)

##################################
include $(CLEAR_VARS)

LOCAL_MODULE := cil_sepolicy.recovery
LOCAL_MODULE_CLASS := ETC
LOCAL_MODULE_TAGS := eng

include $(BUILD_SYSTEM)/base_rules.mk

cil_file_contexts_recovery := $(intermediates)/file_contexts
cil_tunables_recovery := $(intermediates)/tunables.cil

cil_policy_recovery := $(call build_policy, *.cil)

$(LOCAL_BUILT_MODULE) : $(cil_policy_recovery) $(HOST_OUT_EXECUTABLES)/secilc
	@mkdir -p $(dir $@)
	echo "; Auto generated tunables" > $(cil_tunables_recovery)
	echo "(tunable recovery_only true)" >> $(cil_tunables_recovery)
	echo "(tunable force_permissive_to_unconfined $(FORCE_PERMISSIVE_TO_UNCONFINED))" >> $(cil_tunables_recovery)
	echo "(tunable userdebug_or_eng $(USERDEBUG_OR_ENG))" >> $(cil_tunables_recovery)
	$(hide) $(HOST_OUT_EXECUTABLES)/secilc -c $(POLICYVERS) -f $(cil_file_contexts_recovery) -o $@ $(cil_policy_recovery) $(cil_tunables_recovery)

built_sepolicy_recovery := $(LOCAL_BUILT_MODULE)

##################################
include $(CLEAR_VARS)

LOCAL_MODULE := general_sepolicy.cil
LOCAL_MODULE_CLASS := ETC
LOCAL_MODULE_TAGS := tests

include $(BUILD_SYSTEM)/base_rules.mk

cil_tunables_general := $(intermediates)/tunables.cil
cil_dump := $(intermediates)/policy.cil

gen_sepolicy_build_files := $(call build_policy, *.cil)

$(LOCAL_BUILT_MODULE): $(gen_sepolicy_build_files)
	mkdir -p $(dir $@)
	echo "; Auto generated tunables" > $(cil_tunables_general)
	echo "(tunable recovery_only false)" >> $(cil_tunables_general)
	echo "(tunable force_permissive_to_unconfined true)" >> $(cil_tunables_general)
	echo "(tunable userdebug_or_eng false)" >> $(cil_tunables_general)
	m4 -s $(cil_tunables_general) $^ > $(cil_dump)
	sed 's/#/\n;/g' $(cil_dump) > $@
	sed '/dontaudit/d' $@ > $@.dontaudit

GENERAL_SEPOLICY_POLICY.CIL = $(LOCAL_BUILT_MODULE)

########## Build seapp_contexts file ############
include $(CLEAR_VARS)
LOCAL_MODULE := cil_seapp_contexts
LOCAL_MODULE_CLASS := ETC
LOCAL_MODULE_TAGS := optional

include $(BUILD_SYSTEM)/base_rules.mk

cil_seapp_contexts.tmp := $(intermediates)/seapp_contexts.tmp

$(cil_seapp_contexts.tmp) : $(call build_policy, seapp_contexts)
	@mkdir -p $(dir $@)
	m4 -s $^ > $@

$(LOCAL_BUILT_MODULE) : PRIVATE_SEPOLICY := $(built_sepolicy)
$(LOCAL_BUILT_MODULE) : $(cil_seapp_contexts.tmp) $(built_sepolicy) $(HOST_OUT_EXECUTABLES)/checkseapp
	@mkdir -p $(dir $@)
	$(hide) $(HOST_OUT_EXECUTABLES)/checkseapp -p $(PRIVATE_SEPOLICY) -o $@ $<
	@cp $@ $(TARGET_ROOT_OUT)/seapp_contexts

cil_seapp_contexts.tmp :=

########## Build property_contexts file ############
include $(CLEAR_VARS)

LOCAL_MODULE := cil_property_contexts
LOCAL_MODULE_CLASS := ETC
LOCAL_MODULE_TAGS := optional

include $(BUILD_SYSTEM)/base_rules.mk

ALL_CIL_PC_FILES := $(call build_policy, property_contexts)

$(LOCAL_BUILT_MODULE) : PRIVATE_SEPOLICY := $(built_sepolicy)
$(LOCAL_BUILT_MODULE) :  $(ALL_CIL_PC_FILES) $(built_sepolicy) $(HOST_OUT_EXECUTABLES)/checkfc
	@mkdir -p $(dir $@)
	$(hide) m4 -s $(ALL_CIL_PC_FILES) > $@
	$(hide) $(HOST_OUT_EXECUTABLES)/checkfc -p $(PRIVATE_SEPOLICY) $@
	@cp $@ $(TARGET_ROOT_OUT)/property_contexts


########## Build service_contexts file ############
include $(CLEAR_VARS)

LOCAL_MODULE := cil_service_contexts
LOCAL_MODULE_CLASS := ETC
LOCAL_MODULE_TAGS := optional

include $(BUILD_SYSTEM)/base_rules.mk

ALL_CIL_SC_FILES := $(call build_policy, service_contexts)

$(LOCAL_BUILT_MODULE) : PRIVATE_SEPOLICY := $(built_sepolicy)
$(LOCAL_BUILT_MODULE) :  $(ALL_CIL_SC_FILES) $(built_sepolicy) $(HOST_OUT_EXECUTABLES)/checkfc
	@mkdir -p $(dir $@)
	$(hide) m4 -s $(ALL_CIL_SC_FILES) > $@
	$(hide) $(HOST_OUT_EXECUTABLES)/checkfc -p $(PRIVATE_SEPOLICY) $@
	@cp $@ $(TARGET_ROOT_OUT)/service_contexts

##################################
include $(CLEAR_VARS)

LOCAL_MODULE := cil_selinux_version
LOCAL_MODULE_CLASS := ETC
LOCAL_MODULE_TAGS := optional
LOCAL_MODULE_PATH := $(TARGET_ROOT_OUT)

include $(BUILD_SYSTEM)/base_rules.mk
$(LOCAL_BUILT_MODULE) : $(built_sepolicy) $(built_pc) $(built_fc) $(built_sc) $(built_svc)
	@mkdir -p $(dir $@)
	$(hide) echo -n $(BUILD_FINGERPRINT) > $@
	@cp $@ $(TARGET_ROOT_OUT)/selinux_version

##################################

build_policy :=
sepolicy_replace_paths :=
built_sepolicy :=
built_sc :=
built_fc :=
built_pc :=
built_svc :=

include $(call all-makefiles-under,$(LOCAL_PATH))
