#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <selinux/selinux.h>

/* Get entries from $HOME/notebook.conf file */
extern void get_config_entry(char *entry, char **content);

int main(int argc, char **argv)
{
    int rc, fd;
    security_context_t context;
    security_context_t new_context;
    char *path;

#ifdef INFO
    printf("\nThe fsetfilecon example requires a file or directory to be "
                "selected.\nThis file will then opened to obtain a file "
                "descriptor that will be used by\nfgetfilecon to retrieve and "
                "display the context from the xattr\n\"security.selinux\" "
                "using the fgetxattr(2) system call.\n"
                "If okay, a new context will then need to be selected that "
				"will be set by\nfsetfilecon using the fsetxattr(2) call.\n");

    printf("\nNote 1: In permissive mode it is possible to set any context "
                "not defined in the\npolicy (such as \"RUBBISH\"). If this is "
                "done, then in permissive mode\nfgetfilecon will show the "
                "xattr value (i.e. RUBBISH), however in enforcing mode it will "
                "be shown as unlabeled_t if using the Reference Policy.\n");

	printf("\nNote 2: If a valid policy context is set in permissive mode BUT "
                "not allowed for\nthe current process by the policy, then in "
				"enforcement mode the context will be\ndisplayed correctly, "
				"however it will not be possible to change it.\n");

	printf("\nNote 3: Add the mac_admin_example.conf policy module to "
				"experiment with\n'deferred mapping of security contexts' "
				"(but remove module afterwards)."
				"\nPress return to continue\n");
    getchar();
#endif

    get_config_entry("[path]", &path);

    if ((fd = open(path, O_RDONLY)) == -1) {
        printf("Cannot open %s\n", path);
        perror("file open - ERROR");
        exit(1);
    }
    printf("\nOpened file: %s\n", path);

    printf("\nNow obtaining the file context via fgetfilecon:\n");
    if ((rc = fgetfilecon_raw(fd, &context)) == -1) {
        perror("fgetfilecon - ERROR");
        exit(1);
    }
    printf("The current \"security.selinux\" xattr context assigned to the "
                "file is:\n\t%s length: %d\n", context, rc);
    freecon(context);

    printf("\nNow select a new context\nPress return to continue\n");
    getchar();

    get_config_entry("[raw_context]", &new_context);

    printf("\nExecuting: fsetfilecon to set the xattr context to: %s\n",
                                                            new_context);
    if ((rc = fsetfilecon_raw(fd, new_context)) == -1) {
        perror("fsetfilecon - ERROR");
        freecon(new_context);
        exit(1);
    }
    freecon(new_context);
    close(fd);
    free(path);
    exit(0);
}
